@@ -33,6 +33,7 @@
 #'   this value will be returned, while all values less than this value will be set to NA.
 #'   Must be used in conjunction with the argument \code{residmat = TRUE}
 #' @param ... additional arguments to pass
+#'
 #' @author Phil Chalmers \email{rphilip.chalmers@@gmail.com}
 #' @references
 #' Cai, L. & Hansen, M. (2013). Limited-information goodness-of-fit testing of
@@ -75,7 +76,7 @@ M2 <- function(obj, calcNull = TRUE, quadpts = NULL, theta_lim = c(-6, 6),
     #if MG loop
     if(missing(obj)) missingMsg('obj')
     if(is(obj, 'MixedClass'))
-        stop('mixedmirt objects not yet supported', call.=FALSE)
+        stop('MixedClass objects are not yet supported', call.=FALSE)
     if(QMC && is.null(quadpts)) quadpts <- 15000L
     discrete <- FALSE
     if(is(obj, 'DiscreteClass')){
