@@ -1,7 +1,7 @@
 PrepData <- function(data, model, itemtype, guess, upper, gpcm_mats,
                      parprior, verbose, technical, parnumber = 1, BFACTOR = FALSE,
                      grsm.block = NULL, rsm.block = NULL, mixed.design, customItems,
-                     fulldata = NULL, key, nominal.highlow)
+                     fulldata = NULL, key, nominal.highlow, internal_constraints)
 {
     if(is.null(grsm.block)) grsm.block <- rep(1, ncol(data))
     # if(is.null(rsm.block)) rsm.block <- rep(1, ncol(data))
@@ -131,46 +131,68 @@ PrepData <- function(data, model, itemtype, guess, upper, gpcm_mats,
     }
     #within group constraints
     constrain <- list()
-    if(any(itemtype == 'grsm')){
-        unique.grsmgroups <- unique(na.omit(grsm.block))
-        for(group in unique.grsmgroups){
-            Kk <- unique(K[grsm.block == unique.grsmgroups[group]])
-            if(length(Kk) > 1L) stop(c('Rating scale models require items to have the ',
-                                       'same number of categories'), call.=FALSE)
-            for(k in 1L:(Kk-1L)){
-                grsmConstraint <- c()
-                for(i in 1L:J){
-                    if(grsm.block[i] == unique.grsmgroups[group]){
-                        if(length(grsmConstraint) == 0L){
-                            pars[[i]]@est[length(pars[[i]]@est)] <- FALSE
-                            grsmConstraint <- c(grsmConstraint, pars[[i]]@parnum[length(pars[[i]]@parnum)-k])
-                        } else grsmConstraint <- c(grsmConstraint, pars[[i]]@parnum[length(pars[[i]]@parnum)-k])
+    if(internal_constraints){
+        if(any(itemtype == 'grsm')){
+            unique.grsmgroups <- unique(na.omit(grsm.block))
+            for(group in unique.grsmgroups){
+                Kk <- unique(K[grsm.block == unique.grsmgroups[group]])
+                if(length(Kk) > 1L) stop(c('Rating scale models require items to have the ',
+                                           'same number of categories'), call.=FALSE)
+                for(k in 1L:(Kk-1L)){
+                    grsmConstraint <- c()
+                    for(i in 1L:J){
+                        if(grsm.block[i] == unique.grsmgroups[group] && itemtype[i] == 'grsm'){
+                            if(length(grsmConstraint) == 0L){
+                                pars[[i]]@est[length(pars[[i]]@est)] <- FALSE
+                                grsmConstraint <- c(grsmConstraint, pars[[i]]@parnum[length(pars[[i]]@parnum)-k])
+                            } else grsmConstraint <- c(grsmConstraint, pars[[i]]@parnum[length(pars[[i]]@parnum)-k])
+                        }
                     }
+                    constrain[[length(constrain) + 1L]] <- grsmConstraint
                 }
-                constrain[[length(constrain) + 1L]] <- grsmConstraint
             }
         }
-    }
-#     if(any(itemtype == 'rsm')){
-#         unique.rsmgroups <- unique(na.omit(rsm.block))
-#         for(group in unique.rsmgroups){
-#             Kk <- unique(K[rsm.block[rsm.block == unique.rsmgroups[group]]])
-#             if(length(Kk) > 1L) stop('Rating scale models require that items to have the
-#                                     same number of categories', call.=FALSE)
-#             for(k in 1L:(Kk-1L)){
-#                 rsmConstraint <- c()
-#                 for(i in 1L:J){
-#                     if(rsm.block[i] == unique.rsmgroups[group]){
-#                         if(length(rsmConstraint) == 0L){
-#                             pars[[i]]@est[length(pars[[i]]@est)] <- FALSE
-#                             rsmConstraint <- c(rsmConstraint, pars[[i]]@parnum[length(pars[[i]]@parnum)-k])
-#                         } else rsmConstraint <- c(rsmConstraint, pars[[i]]@parnum[length(pars[[i]]@parnum)-k])
+        if(any(itemtype == 'grsmIRT')){
+            unique.grsmgroups <- unique(na.omit(grsm.block))
+            for(group in unique.grsmgroups){
+                Kk <- unique(K[grsm.block == unique.grsmgroups[group]])
+                if(length(Kk) > 1L) stop(c('Rating scale models require items to have the ',
+                                           'same number of categories'), call.=FALSE)
+                for(k in 1L:(Kk-1L)){
+                    grsmConstraint <- c()
+                    for(i in 1L:J){
+                        if(grsm.block[i] == unique.grsmgroups[group] && itemtype[i] == 'grsmIRT'){
+                            if(length(grsmConstraint) == 0L){
+                                pars[[i]]@est[length(pars[[i]]@est)] <- FALSE
+                                grsmConstraint <- c(grsmConstraint, pars[[i]]@parnum[length(pars[[i]]@parnum)-k])
+                            } else grsmConstraint <- c(grsmConstraint, pars[[i]]@parnum[length(pars[[i]]@parnum)-k])
+                        }
+                    }
+                    constrain[[length(constrain) + 1L]] <- grsmConstraint
+                }
+            }
+        }
+#         if(any(itemtype == 'rsm')){
+#             unique.rsmgroups <- unique(na.omit(rsm.block))
+#             for(group in unique.rsmgroups){
+#                 Kk <- unique(K[rsm.block[rsm.block == unique.rsmgroups[group]]])
+#                 if(length(Kk) > 1L) stop('Rating scale models require that items to have the
+#                                         same number of categories', call.=FALSE)
+#                 for(k in 1L:(Kk-1L)){
+#                     rsmConstraint <- c()
+#                     for(i in 1L:J){
+#                         if(rsm.block[i] == unique.rsmgroups[group] && itemtype[i] == 'rsm'){
+#                             if(length(rsmConstraint) == 0L){
+#                                 pars[[i]]@est[length(pars[[i]]@est)] <- FALSE
+#                                 rsmConstraint <- c(rsmConstraint, pars[[i]]@parnum[length(pars[[i]]@parnum)-k])
+#                             } else rsmConstraint <- c(rsmConstraint, pars[[i]]@parnum[length(pars[[i]]@parnum)-k])
+#                         }
 #                     }
+#                     constrain[[length(constrain) + 1L]] <- rsmConstraint
 #                 }
-#                 constrain[[length(constrain) + 1L]] <- rsmConstraint
 #             }
 #         }
-#     }
+    }
     npars <- sum(sapply(pars, function(x) sum(x@est)))
     if(is.null(prodlist)) prodlist <- list()
     ret <- list(pars=pars, npars=npars, constrain=constrain, prodlist=prodlist, itemnames=itemnames,
