@@ -9,6 +9,8 @@
 #'   Only applicable when the input object is multidimensional
 #' @param group a number signifying which group the item should be extracted from (applies to
 #'   'MultipleGroupClass' objects only)
+#'
+#' @author Phil Chalmers \email{rphilip.chalmers@@gmail.com}
 #' @keywords information
 #' @export testinfo
 #' @examples
@@ -40,7 +42,7 @@ testinfo <- function(x, Theta, degrees = NULL, group = NULL){
     if(missing(x)) missingMsg('x')
     if(missing(Theta)) missingMsg('Theta')
     if(!is.matrix(Theta)) Theta <- as.matrix(Theta)
-    J <- x@Data$nitems
+    J <- extract.mirt(x, 'nitems')
     info <- 0
     for(i in 1L:J){
         item <- extract.item(x, i, group=group)
